<?php

namespace FiloBlu\Esb\Setup;

use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Exception\AlreadyExistsException;

/**
 * Class UpgradeData
 * @package FiloBlu\Esb\Setup
 */
class UpgradeData implements UpgradeDataInterface {
    const STATE_HOLDED = 'holded';
    const STATE_NEW = 'new';
    const STATE_PAYMENT_REVIEW = 'payment_review';
    const STATE_PENDING_PAYMENT = 'pending_payment';

    /**
     * @var \Magento\Sales\Model\Order\StatusFactory
     */
    protected $_statusFactory;

    /**
     * InstallData constructor.
     * @param \Magento\Sales\Model\Order\StatusFactory $statusFactory
     */
    public function __construct(
        \Magento\Sales\Model\Order\StatusFactory $statusFactory
    )
    {
        $this->_statusFactory = $statusFactory;
    }

    /**
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     * @throws \Exception
     */
    public function upgrade( ModuleDataSetupInterface $setup, ModuleContextInterface $context ) {

        $setup->startSetup();
        
        if (version_compare($context->getVersion(), '1.0.2') < 0) {
            $confirmedStatus = $this->_statusFactory->create();
            $confirmedStatus->setData('status', 'confirmed');
            $confirmedStatus->setData('label', 'Confirmed');
            $confirmedStatus->save();
            $confirmedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);

            $orderExportedStatus = $this->_statusFactory->create();
            $orderExportedStatus->setData('status', 'order_exported');
            $orderExportedStatus->setData('label', 'Order Exported');
            $orderExportedStatus->save();
            $orderExportedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, false);

            $paymentConfirmedStatus = $this->_statusFactory->create();
            $paymentConfirmedStatus->setData('status', 'payment_confirmed');
            $paymentConfirmedStatus->setData('label', 'Payment Confirmed');
            $paymentConfirmedStatus->save();
            $paymentConfirmedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);

            $paymentExportedStatus = $this->_statusFactory->create();
            $paymentExportedStatus->setData('status', 'payment_exported');
            $paymentExportedStatus->setData('label', 'Payment Exported');
            $paymentExportedStatus->save();
            $paymentExportedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, false);

            $invoicedStatus = $this->_statusFactory->create();
            $invoicedStatus->setData('status', 'invoiced');
            $invoicedStatus->setData('label', 'Invoiced');
            $invoicedStatus->save();
            $invoicedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);

            $shippedStatus = $this->_statusFactory->create();
            $shippedStatus->setData('status', 'shipped');
            $shippedStatus->setData('label', 'Shipped');
            $shippedStatus->save();
            $shippedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);
        }

        if (version_compare($context->getVersion(), '1.0.21') < 0) {
            $newStatus = $this->_statusFactory->create();
            $newStatus->setData('status', 'sent_to_logistics');
            $newStatus->setData('label', 'Sent to Logistics');
            $newStatus->save();
            $newStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);
        }

        if(version_compare($context->getVersion(), '1.1.9') < 0) {
            $newStatus = $this->_statusFactory->create();
            $newStatus->setData('status', 'blocked');
            $newStatus->setData('label', 'Blocked');
            try{
                $newStatus->save();
            } catch (AlreadyExistsException $e) {}

            try{
                $newStatus->assignState(self::STATE_HOLDED, false, true);
            }catch (AlreadyExistsException $e){}

            try{
                $newStatus->assignState(self::STATE_NEW, false, true);
            }catch (AlreadyExistsException $e){}

            try{
                $newStatus->assignState(self::STATE_PAYMENT_REVIEW, false, true);
            }catch (AlreadyExistsException $e){}

            try{
                $newStatus->assignState(self::STATE_PENDING_PAYMENT, false, true);
            }catch (AlreadyExistsException $e){}
        }
    }
}